require_relative './test_helper'
require 'adorn'
require 'adorn/cache'

ImportantObject = Struct.new(:name, :age) do
  def name
    print "my name is #{@name}"
  end

  def age
    @age
  end

  def kindergarten_greeting
    puts "Hi #{name} and I am #{age}."
  end
end

class TestAdornedCache < Adorn::TestCase
  def setup
  end

  def teardown
    Adorn::Cache.clear!
  end

  def test_insert_key
    me_or_someone_like_me = ImportantObject.new('rhodee', 42)
    Adorn::Cache.append! :person, me_or_someone_like_me

    refute_equal true, Thread.current[:adorn_cache].keys.empty?
    assert_same me_or_someone_like_me, Thread.current[:adorn_cache]['person']
  end

  def test_read_key
    me_or_someone_like_me = ImportantObject.new('rhodee', 42)
    Adorn::Cache.append! :person, me_or_someone_like_me

    assert_same me_or_someone_like_me, Thread.current[:adorn_cache]['person']
    assert_equal me_or_someone_like_me, Adorn::Cache[:person]
  end

  def test_read_unknown_key
    assert_raises(NoMethodError) { Adorn::Cache[:unknown_key] }
  end

  def test_clear_key
    another_person = ImportantObject.new('snuggs', 10)
    Adorn::Cache.append! :peep, another_person

    refute_equal true, Thread.current[:adorn_cache].keys.empty?

    Adorn::Cache.clear!

    assert_equal true, Thread.current[:adorn_cache].empty?
  end

  def test_cache_objects
    me_or_someone_like_me = ImportantObject.new('rhodee', 42)
    Adorn::Cache.append! :person, me_or_someone_like_me

    4.times do
      Thread.new do
        me_or_someone_like_me = ImportantObject.new('rhodee', 42)
        Adorn::Cache.append! :person, me_or_someone_like_me
      end
    end

    assert_equal me_or_someone_like_me.object_id, Thread.current[:adorn_cache]['person'].object_id
  end
end
