require_relative './test_helper'
require 'adorn'

AdornedWithUsefulMethods = Module.new do
  def calculate; return 1 + 2; end
end

MoreUsefulMethods = Module.new do
  def subtract; return 1 - 2; end
end

DecoratedClass = Struct.new(:time)
DummyContext = Struct.new(:path)

class Proxy < Adorn::Presenter
  Adorn::Delegate.extensions { {with: [AdornedWithUsefulMethods, MoreUsefulMethods]} }
  presents :model

  def model_value
    print "That's a big number #{model.calculate}"
  end
end

class TestAdornedObject < Adorn::TestCase
  def setup
    @obj_proxy     = DecoratedClass.new(Time.now)
    @dummy_context = DummyContext.new('path/to/context')
  end

  def teardown
    @obj_proxy     = nil
    @dummy_context = nil
  end

  def test_present_helper
    proxy = Proxy.new(@obj_proxy, @dummy_context)

    assert true, proxy.model.is_a?(DecoratedClass)
  end

  def test_object_extension
    proxy = Proxy.new(@obj_proxy, @dummy_context)

    assert_includes proxy.model.methods, :calculate
    assert_includes proxy.model.methods, :subtract
  end

  def test_method_lookup_error
    proxy = Proxy.new(@obj_proxy, @dummy_context)

    assert_raises(NoMethodError) { proxy.not_a_method }
  end

  def test_decoration
    assert_output("That's a big number 3") { Proxy.new(@obj_proxy, @dummy_context).model_value }
  end
end
